@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2019 - 2022, Ilan Schnell; All Rights Reserved
+   Copyright (c) 2019 - 2023, Ilan Schnell; All Rights Reserved
    bitarray is published under the PSF license.
 
    This file contains the C implementation of some useful utility functions.
@@ -15,7 +15,7 @@
 /* set during module initialization */
 static PyObject *bitarray_type_obj;
 
-/* Return 0 if obj is bitarray.  If not, return -1 and set an exception. */
+/* Return 0 if obj is bitarray.  If not, set exception and return -1. */
 static int
 ensure_bitarray(PyObject *obj)
 {
@@ -34,8 +34,8 @@ ensure_bitarray(PyObject *obj)
 
 /* ------------------------------- count_n ----------------------------- */
 
-/* return the smallest index i for which a.count(vi, 0, i) == n, or when
-   n exceeds the total count return -1  */
+/* Return the smallest index i for which a.count(vi, 0, i) == n.
+   When n exceeds the total count, return -1.  */
 static Py_ssize_t
 count_to_n(bitarrayobject *a, Py_ssize_t n, int vi)
 {
@@ -93,25 +93,23 @@ count_to_n(bitarrayobject *a, Py_ssize_t n, int vi)
 static PyObject *
 count_n(PyObject *module, PyObject *args)
 {
-    PyObject *a;
+    bitarrayobject *a;
     Py_ssize_t n, i;
     int vi = 1;
 
-    if (!PyArg_ParseTuple(args, "O!n|O&:count_n",
-                          bitarray_type_obj, &a, &n, conv_pybit, &vi))
+    if (!PyArg_ParseTuple(args, "O!n|O&:count_n", bitarray_type_obj,
+                          (PyObject *) &a, &n, conv_pybit, &vi))
         return NULL;
     if (n < 0) {
         PyErr_SetString(PyExc_ValueError, "non-negative integer expected");
         return NULL;
     }
-
-#define aa  ((bitarrayobject *) a)
-    if (n > aa->nbits)  {
+    if (n > a->nbits)  {
         PyErr_SetString(PyExc_ValueError, "n larger than bitarray size");
         return NULL;
     }
-    i = count_to_n(aa, n, vi);        /* do actual work here */
-#undef aa
+    i = count_to_n(a, n, vi);        /* do actual work here */
+
     if (i < 0) {
         PyErr_SetString(PyExc_ValueError, "n exceeds total count");
         return NULL;
@@ -183,19 +181,16 @@ find_last(bitarrayobject *self, int vi, Py_ssize_t a, Py_ssize_t b)
 static PyObject *
 r_index(PyObject *module, PyObject *args)
 {
-    PyObject *a;
+    bitarrayobject *a;
     Py_ssize_t start = 0, stop = PY_SSIZE_T_MAX, res;
     int vi = 1;
 
-    if (!PyArg_ParseTuple(args, "O!|O&nn:rindex", bitarray_type_obj, &a,
-                          conv_pybit, &vi, &start, &stop))
+    if (!PyArg_ParseTuple(args, "O!|O&nn:rindex", bitarray_type_obj,
+                          (PyObject *) &a, conv_pybit, &vi, &start, &stop))
         return NULL;
 
-#define aa  ((bitarrayobject *) a)
-    adjust_indices(aa->nbits, &start, &stop, 1);
-    res = find_last(aa, vi, start, stop);
-#undef aa
-    if (res < 0)
+    adjust_indices(a->nbits, &start, &stop, 1);
+    if ((res = find_last(a, vi, start, stop)) < 0)
         return PyErr_Format(PyExc_ValueError, "%d not in bitarray", vi);
 
     return PyLong_FromSsize_t(res);
@@ -210,20 +205,20 @@ Raises `ValueError` if the value is not present.");
 /* --------------------------- unary functions ------------------------- */
 
 static PyObject *
-parity(PyObject *module, PyObject *a)
+parity(PyObject *module, PyObject *obj)
 {
+    bitarrayobject *a;
     unsigned char par = 0;
     Py_ssize_t i;
 
-    if (ensure_bitarray(a) < 0)
+    if (ensure_bitarray(obj) < 0)
         return NULL;
 
-#define aa  ((bitarrayobject *) a)
-    for (i = 0; i < aa->nbits / 8; i++)
-        par ^= aa->ob_item[i];
-    if (aa->nbits % 8)
-        par ^= zeroed_last_byte(aa);
-#undef aa
+    a = (bitarrayobject *) obj;
+    for (i = 0; i < a->nbits / 8; i++)
+        par ^= a->ob_item[i];
+    if (a->nbits % 8)
+        par ^= zeroed_last_byte(a);
 
     return PyLong_FromLong((long) bitcount_lookup[par] % 2);
 }
@@ -232,111 +227,85 @@ PyDoc_STRVAR(parity_doc,
 "parity(a, /) -> int\n\
 \n\
 Return the parity of bitarray `a`.\n\
-This is equivalent to `a.count() % 2` (but more efficient).");
+`parity(a)` is equivalent to `a.count() % 2` but more efficient.");
 
 /* --------------------------- binary functions ------------------------ */
 
-enum kernel_type {
-    KERN_cand,     /* count bitwise and -> int */
-    KERN_cor,      /* count bitwise or -> int */
-    KERN_cxor,     /* count bitwise xor -> int */
-    KERN_subset,   /* is subset -> bool */
-};
-
 static PyObject *
-binary_function(PyObject *args, enum kernel_type kern, const char *format)
+binary_function(PyObject *args, const char *format, const char oper)
 {
-    Py_ssize_t res = 0, s, i;
-    PyObject *a, *b;
-    char *buff_a, *buff_b;
-    unsigned char c;
+    Py_ssize_t cnt = 0, s, i;
+    bitarrayobject *a, *b;
+    unsigned char *buff_a, *buff_b;
     int r;
 
     if (!PyArg_ParseTuple(args, format,
-                          bitarray_type_obj, &a, bitarray_type_obj, &b))
+                          bitarray_type_obj, (PyObject *) &a,
+                          bitarray_type_obj, (PyObject *) &b))
         return NULL;
 
-#define aa  ((bitarrayobject *) a)
-#define bb  ((bitarrayobject *) b)
-    if (aa->nbits != bb->nbits) {
+    if (a->nbits != b->nbits) {
         PyErr_SetString(PyExc_ValueError,
                         "bitarrays of equal length expected");
         return NULL;
     }
-    if (aa->endian != bb->endian) {
+    if (a->endian != b->endian) {
         PyErr_SetString(PyExc_ValueError,
                         "bitarrays of equal endianness expected");
         return NULL;
     }
-    buff_a = aa->ob_item;
-    buff_b = bb->ob_item;
-    s = aa->nbits / 8;       /* number of whole bytes in buffer */
-    r = aa->nbits % 8;       /* remaining bits  */
+    buff_a = (unsigned char *) a->ob_item;
+    buff_b = (unsigned char *) b->ob_item;
+    s = a->nbits / 8;       /* number of whole bytes in buffer */
+    r = a->nbits % 8;       /* remaining bits  */
 
-    switch (kern) {
-    case KERN_cand:
-        for (i = 0; i < s; i++) {
-            c = buff_a[i] & buff_b[i];
-            res += bitcount_lookup[c];
-        }
-        if (r) {
-            c = zeroed_last_byte(aa) & zeroed_last_byte(bb);
-            res += bitcount_lookup[c];
-        }
+    switch (oper) {
+#define UZ(x)  ((unsigned char) zeroed_last_byte(x))
+    case '&':                   /* count and */
+        for (i = 0; i < s; i++)
+            cnt += bitcount_lookup[buff_a[i] & buff_b[i]];
+        if (r)
+            cnt += bitcount_lookup[UZ(a) & UZ(b)];
         break;
 
-    case KERN_cor:
-        for (i = 0; i < s; i++) {
-            c = buff_a[i] | buff_b[i];
-            res += bitcount_lookup[c];
-        }
-        if (r) {
-            c = zeroed_last_byte(aa) | zeroed_last_byte(bb);
-            res += bitcount_lookup[c];
-        }
+    case '|':                   /* count or */
+        for (i = 0; i < s; i++)
+            cnt += bitcount_lookup[buff_a[i] | buff_b[i]];
+        if (r)
+            cnt += bitcount_lookup[UZ(a) | UZ(b)];
         break;
 
-    case KERN_cxor:
-        for (i = 0; i < s; i++) {
-            c = buff_a[i] ^ buff_b[i];
-            res += bitcount_lookup[c];
-        }
-        if (r) {
-            c = zeroed_last_byte(aa) ^ zeroed_last_byte(bb);
-            res += bitcount_lookup[c];
-        }
+    case '^':                   /* count xor */
+        for (i = 0; i < s; i++)
+            cnt += bitcount_lookup[buff_a[i] ^ buff_b[i]];
+        if (r)
+            cnt += bitcount_lookup[UZ(a) ^ UZ(b)];
         break;
 
-    case KERN_subset:
+    case 's':                   /* is subset */
         for (i = 0; i < s; i++) {
             if ((buff_a[i] & buff_b[i]) != buff_a[i])
                 Py_RETURN_FALSE;
         }
-        if (r) {
-            if ((zeroed_last_byte(aa) & zeroed_last_byte(bb)) !=
-                 zeroed_last_byte(aa))
-                Py_RETURN_FALSE;
-        }
-        Py_RETURN_TRUE;
+        return PyBool_FromLong(r == 0 || (UZ(a) & UZ(b)) == UZ(a));
 
     default:
         Py_UNREACHABLE();
+#undef UZ
     }
-#undef aa
-#undef bb
-    return PyLong_FromSsize_t(res);
+    return PyLong_FromSsize_t(cnt);
 }
 
-#define COUNT_FUNC(oper, ochar)                                         \
+#define COUNT_FUNC(oper, ostr)                                          \
 static PyObject *                                                       \
 count_ ## oper (PyObject *module, PyObject *args)                       \
 {                                                                       \
-    return binary_function(args, KERN_c ## oper, "O!O!:count_" #oper);  \
+    return binary_function(args, "O!O!:count_" #oper, *ostr);           \
 }                                                                       \
 PyDoc_STRVAR(count_ ## oper ## _doc,                                    \
 "count_" #oper "(a, b, /) -> int\n\
 \n\
-Return `(a " ochar " b).count()` in a memory efficient manner,\n\
+Return `(a " ostr " b).count()` in a memory efficient manner,\n\
 as no intermediate bitarray object gets created.")
 
 COUNT_FUNC(and, "&");           /* count_and */
@@ -347,16 +316,16 @@ COUNT_FUNC(xor, "^");           /* count_xor */
 static PyObject *
 subset(PyObject *module, PyObject *args)
 {
-    return binary_function(args, KERN_subset, "O!O!:subset");
+    return binary_function(args, "O!O!:subset", 's');
 }
 
 PyDoc_STRVAR(subset_doc,
 "subset(a, b, /) -> bool\n\
 \n\
 Return `True` if bitarray `a` is a subset of bitarray `b`.\n\
-`subset(a, b)` is equivalent to `a | b == b` (and `a & b == a`) but more\n\
-efficient as iterating the buffers can be stopped as soon as one mismatch\n\
-is found.  Moreover, no intermediate bitarray object is created.");
+`subset(a, b)` is equivalent to `a | b == b` (and equally `a & b == a`) but\n\
+more efficient as no intermediate bitarray object is created and the buffer\n\
+iteration is stopped as soon as one mismatch found.");
 
 /* ---------------------------- serialization -------------------------- */
 
@@ -407,41 +376,41 @@ hex_to_int(char c)
 }
 
 static PyObject *
-ba2hex(PyObject *module, PyObject *a)
+ba2hex(PyObject *module, PyObject *obj)
 {
     PyObject *result;
+    bitarrayobject *a;
     size_t i, strsize;
     char *str;
     int le, be;
 
-    if (ensure_bitarray(a) < 0)
+    if (ensure_bitarray(obj) < 0)
         return NULL;
 
-#define aa  ((bitarrayobject *) a)
-    if (aa->nbits % 4) {
+    a = (bitarrayobject *) obj;
+    if (a->nbits % 4) {
         PyErr_SetString(PyExc_ValueError, "bitarray length not multiple of 4");
         return NULL;
     }
 
     /* We want strsize to be even, such that we can transform the entire
-       bitarray buffer at once.  Hence, we don't use aa->nbits / 4 here, as
+       bitarray buffer at once.  Hence, we don't use a->nbits / 4 here, as
        is could make strsize odd. */
-    strsize = 2 * Py_SIZE(a);
+    strsize = 2 * Py_SIZE(obj);
     str = (char *) PyMem_Malloc(strsize);
     if (str == NULL)
         return PyErr_NoMemory();
 
-    le = IS_LE(aa);
-    be = IS_BE(aa);
+    le = IS_LE(a);
+    be = IS_BE(a);
     for (i = 0; i < strsize; i += 2) {
-        unsigned char c = aa->ob_item[i / 2];
+        unsigned char c = a->ob_item[i / 2];
 
         str[i + le] = hexdigits[c >> 4];
         str[i + be] = hexdigits[0x0f & c];
     }
-    assert((size_t) aa->nbits / 4 <= strsize);
-    result = Py_BuildValue("s#", str, aa->nbits / 4);
-#undef aa
+    assert((size_t) a->nbits / 4 <= strsize);
+    result = Py_BuildValue("s#", str, a->nbits / 4);
     PyMem_Free((void *) str);
     return result;
 }
@@ -459,21 +428,21 @@ the bitarray (which has to be multiple of 4 in length).");
 static PyObject *
 hex2ba(PyObject *module, PyObject *args)
 {
-    PyObject *a;
+    bitarrayobject *a;
     char *str;
     Py_ssize_t i, strsize;
     int le, be;
 
-    if (!PyArg_ParseTuple(args, "O!s#", bitarray_type_obj, &a, &str, &strsize))
+    if (!PyArg_ParseTuple(args, "O!s#", bitarray_type_obj, (PyObject *) &a,
+                          &str, &strsize))
         return NULL;
 
-#define aa  ((bitarrayobject *) a)
-    if (aa->nbits != 4 * strsize) {
+    if (a->nbits != 4 * strsize) {
         PyErr_SetString(PyExc_ValueError, "size mismatch");
         return NULL;
     }
-    le = IS_LE(aa);
-    be = IS_BE(aa);
+    le = IS_LE(a);
+    be = IS_BE(a);
     assert(le + be == 1 && str[strsize] == 0);
     for (i = 0; i < strsize; i += 2) {
         int x = hex_to_int(str[i + le]);
@@ -493,9 +462,8 @@ hex2ba(PyObject *module, PyObject *args)
             }
         }
         assert(0 <= x && x < 16 && 0 <= y && y < 16);
-        aa->ob_item[i / 2] = x << 4 | y;
+        a->ob_item[i / 2] = x << 4 | y;
     }
-#undef aa
     Py_RETURN_NONE;
 }
 
@@ -561,12 +529,14 @@ static PyObject *
 ba2base(PyObject *module, PyObject *args)
 {
     const char *alphabet;
-    PyObject *result, *a;
+    bitarrayobject *a;
+    PyObject *result;
     size_t i, strsize;
     char *str;
     int n, m, le;
 
-    if (!PyArg_ParseTuple(args, "iO!:ba2base", &n, bitarray_type_obj, &a))
+    if (!PyArg_ParseTuple(args, "iO!:ba2base", &n,
+                          bitarray_type_obj, (PyObject *) &a))
         return NULL;
     if ((m = base_to_length(n)) < 0)
         return NULL;
@@ -577,27 +547,25 @@ ba2base(PyObject *module, PyObject *args)
     default: alphabet = hexdigits;
     }
 
-#define aa  ((bitarrayobject *) a)
-    if (aa->nbits % m)
+    if (a->nbits % m)
         return PyErr_Format(PyExc_ValueError,
                             "bitarray length must be multiple of %d", m);
 
-    strsize = aa->nbits / m;
+    strsize = a->nbits / m;
     if ((str = (char *) PyMem_Malloc(strsize)) == NULL)
         return PyErr_NoMemory();
 
-    le = IS_LE(aa);
+    le = IS_LE(a);
     for (i = 0; i < strsize; i++) {
         int j, x = 0;
 
         for (j = 0; j < m; j++) {
             int k = le ? j : (m - j - 1);
-            x |= getbit(aa, i * m + k) << j;
+            x |= getbit(a, i * m + k) << j;
         }
         str[i] = alphabet[x];
     }
     result = Py_BuildValue("s#", str, strsize);
-#undef aa
     PyMem_Free((void *) str);
     return result;
 }
@@ -623,25 +591,24 @@ standard base 64 alphabet is used.");
 static PyObject *
 base2ba(PyObject *module, PyObject *args)
 {
-    PyObject *a = NULL;
+    bitarrayobject *a = NULL;
     Py_ssize_t i, strsize = 0;
     char *str = NULL;
     int n, m, le;
 
-    if (!PyArg_ParseTuple(args, "i|O!s#",
-                          &n, bitarray_type_obj, &a, &str, &strsize))
+    if (!PyArg_ParseTuple(args, "i|O!s#", &n, bitarray_type_obj,
+                          (PyObject *) &a, &str, &strsize))
         return NULL;
     if ((m = base_to_length(n)) < 0)
         return NULL;
     if (a == NULL)  /* when only base n is given - return length log2(n) */
         return PyLong_FromLong(m);
 
-#define aa  ((bitarrayobject *) a)
-    if (aa->nbits != m * strsize) {
+    if (a->nbits != m * strsize) {
         PyErr_SetString(PyExc_ValueError, "size mismatch");
         return NULL;
     }
-    le = IS_LE(aa);
+    le = IS_LE(a);
     for (i = 0; i < strsize; i++) {
         int j, d = digit_to_int(n, str[i]);
 
@@ -652,10 +619,9 @@ base2ba(PyObject *module, PyObject *args)
         }
         for (j = 0; j < m; j++) {
             int k = le ? j : (m - j - 1);
-            setbit(aa, i * m + k, d & (1 << j));
+            setbit(a, i * m + k, d & (1 << j));
         }
     }
-#undef aa
     Py_RETURN_NONE;
 }
 
@@ -702,19 +668,20 @@ grow_buffer(bitarrayobject *self)
 static PyObject *
 vl_decode(PyObject *module, PyObject *args)
 {
-    PyObject *iter, *item, *a;
+    PyObject *iter, *item;
+    bitarrayobject *a;
     Py_ssize_t padding = 0;  /* number of pad bits read from header byte */
     Py_ssize_t i = 0;        /* bit counter */
     unsigned char b = 0x80;  /* empty stream will raise StopIteration */
     Py_ssize_t k;
 
-    if (!PyArg_ParseTuple(args, "OO!", &iter, bitarray_type_obj, &a))
+    if (!PyArg_ParseTuple(args, "OO!", &iter,
+                          bitarray_type_obj, (PyObject *) &a))
         return NULL;
     if (!PyIter_Check(iter))
         return PyErr_Format(PyExc_TypeError, "iterator or bytes expected, "
                             "got '%s'", Py_TYPE(iter)->tp_name);
 
-#define aa  ((bitarrayobject *) a)
     while ((item = PyIter_Next(iter))) {
 #ifdef IS_PY3K
         if (PyLong_Check(item))
@@ -731,9 +698,9 @@ vl_decode(PyObject *module, PyObject *args)
         }
         Py_DECREF(item);
 
-        if (i + 6 >= aa->nbits && grow_buffer(aa) < 0)
+        if (i + 6 >= a->nbits && grow_buffer(a) < 0)
             return NULL;
-        assert(i + 6 < aa->nbits);
+        assert(i + 6 < a->nbits);
 
         if (i == 0) {
             padding = (b & 0x70) >> 4;
@@ -741,20 +708,19 @@ vl_decode(PyObject *module, PyObject *args)
                 return PyErr_Format(PyExc_ValueError,
                                     "invalid header byte: 0x%02x", b);
             for (k = 0; k < 4; k++)
-                setbit(aa, i++, (0x08 >> k) & b);
+                setbit(a, i++, (0x08 >> k) & b);
         }
         else {
             for (k = 0; k < 7; k++)
-                setbit(aa, i++, (0x40 >> k) & b);
+                setbit(a, i++, (0x40 >> k) & b);
         }
         if ((b & 0x80) == 0)
             break;
     }
     /* set final length of bitarray */
-    aa->nbits = i - padding;
-    Py_SET_SIZE(a, BYTES(aa->nbits));
-    assert_nbits(aa);
-#undef aa
+    a->nbits = i - padding;
+    Py_SET_SIZE(a, BYTES(a->nbits));
+    assert_nbits(a);
 
     if (PyErr_Occurred())       /* from PyIter_Next() */
         return NULL;
@@ -767,20 +733,21 @@ vl_decode(PyObject *module, PyObject *args)
 }
 
 static PyObject *
-vl_encode(PyObject *module, PyObject *a)
+vl_encode(PyObject *module, PyObject *obj)
 {
     PyObject *result;
+    bitarrayobject *a;
     Py_ssize_t padding, n, m, i;
     Py_ssize_t j = 0;           /* byte conter */
     char *str;
 
-    if (ensure_bitarray(a) < 0)
+    if (ensure_bitarray(obj) < 0)
         return NULL;
 
-#define aa  ((bitarrayobject *) a)
-    n = (aa->nbits + LEN_PAD_BITS + 6) / 7;  /* number of resulting bytes */
+    a = (bitarrayobject *) obj;
+    n = (a->nbits + LEN_PAD_BITS + 6) / 7;  /* number of resulting bytes */
     m = 7 * n - LEN_PAD_BITS;    /* number of bits resulting bytes can hold */
-    padding = m - aa->nbits;     /* number of pad bits */
+    padding = m - a->nbits;      /* number of pad bits */
     assert(0 <= padding && padding < 7);
 
     result = PyBytes_FromStringAndSize(NULL, n);
@@ -788,21 +755,20 @@ vl_encode(PyObject *module, PyObject *a)
         return NULL;
 
     str = PyBytes_AsString(result);
-    str[0] = aa->nbits > 4 ? 0x80 : 0x00;  /* leading bit */
+    str[0] = a->nbits > 4 ? 0x80 : 0x00;   /* leading bit */
     str[0] |= padding << 4;                /* encode padding */
-    for (i = 0; i < 4 && i < aa->nbits; i++)
-        str[0] |= (0x08 >> i) * getbit(aa, i);
+    for (i = 0; i < 4 && i < a->nbits; i++)
+        str[0] |= (0x08 >> i) * getbit(a, i);
 
-    for (i = 4; i < aa->nbits; i++) {
+    for (i = 4; i < a->nbits; i++) {
         int k = (i - 4) % 7;
 
         if (k == 0) {
             j++;
             str[j] = j < n - 1 ? 0x80 : 0x00;  /* leading bit */
         }
-        str[j] |= (0x40 >> k) * getbit(aa, i);
+        str[j] |= (0x40 >> k) * getbit(a, i);
     }
-#undef aa
     assert(j == n - 1);
 
     return result;
