@@ -15,15 +15,16 @@ from string import hexdigits
 from random import choice, randint, random
 from collections import Counter
 
-from bitarray import (bitarray, frozenbitarray, bits2bytes, decodetree,
+from bitarray import (bitarray, frozenbitarray, decodetree,
                       get_default_endian, _set_default_endian)
-from bitarray.test_bitarray import Util
+from bitarray.test_bitarray import Util, skipIf
 
 from bitarray.util import (
     zeros, urandom, pprint, make_endian, rindex, strip, count_n,
     parity, count_and, count_or, count_xor, subset,
     serialize, deserialize, ba2hex, hex2ba, ba2base, base2ba,
-    ba2int, int2ba, vl_encode, vl_decode, huffman_code
+    ba2int, int2ba, vl_encode, vl_decode,
+    huffman_code, canonical_huffman, canonical_decode,
 )
 
 if sys.version_info[0] == 3:
@@ -427,28 +428,43 @@ class TestsCount_N(unittest.TestCase, Util):
             i += 1
         return i
 
-    def check_result(self, a, n, i):
-        self.assertEqual(a.count(1, 0, i), n)
+    def check_result(self, a, n, i, v=1):
+        self.assertEqual(a.count(v, 0, i), n)
         if i == 0:
             self.assertEqual(n, 0)
         else:
-            self.assertTrue(a[i - 1])
+            self.assertEqual(a[i - 1], v)
 
     def test_empty(self):
         a = bitarray()
         self.assertEqual(count_n(a, 0), 0)
+        self.assertEqual(count_n(a, 0, 0), 0)
+        self.assertEqual(count_n(a, 0, 1), 0)
         self.assertRaises(ValueError, count_n, a, 1)
         self.assertRaises(TypeError, count_n, '', 0)
         self.assertRaises(TypeError, count_n, a, 7.0)
-        self.assertRaises(TypeError, count_n, a, 0, 0)
+        self.assertRaises(ValueError, count_n, a, 0, 2)
 
     def test_simple(self):
         a = bitarray('111110111110111110111110011110111110111110111000')
         b = a.copy()
         self.assertEqual(len(a), 48)
         self.assertEqual(a.count(), 37)
+        self.assertEqual(a.count(0), 11)
+
+        self.assertEqual(count_n(a, 0, 0), 0)
+        self.assertEqual(count_n(a, 2, 0), 12)
+        self.assertEqual(count_n(a, 10, 0), 47)
+        self.assertRaisesMessage(ValueError, "non-negative integer expected",
+                                 count_n, a, -1, 0) # n < 0
+        self.assertRaisesMessage(ValueError, "n larger than bitarray size",
+                                 count_n, a, 49, 0) # n > len(a)
+        self.assertRaisesMessage(ValueError, "n exceeds total count",
+                                 count_n, a, 12, 0) # n > a.count(0)
+
         self.assertEqual(count_n(a, 0), 0)
         self.assertEqual(count_n(a, 20), 23)
+        self.assertEqual(count_n(a, 20, 1), 23)
         self.assertEqual(count_n(a, 37), 45)
         self.assertRaisesMessage(ValueError, "non-negative integer expected",
                                  count_n, a, -1) # n < 0
@@ -456,11 +472,13 @@ class TestsCount_N(unittest.TestCase, Util):
                                  count_n, a, 49) # n > len(a)
         self.assertRaisesMessage(ValueError, "n exceeds total count",
                                  count_n, a, 38) # n > a.count()
-        for n in range(0, 38):
-            i = count_n(a, n)
-            self.check_result(a, n, i)
-            self.assertEqual(a[:i].count(), n)
-            self.assertEqual(i, self.count_n(a, n))
+
+        for v in 0, 1:
+            for n in range(a.count(v) + 1):
+                i = count_n(a, n, v)
+                self.check_result(a, n, i, v)
+                self.assertEqual(a[:i].count(v), n)
+                self.assertEqual(i, self.count_n(a if v else ~a, n))
         self.assertEQUAL(a, b)
 
     def test_frozen(self):
@@ -473,7 +491,7 @@ class TestsCount_N(unittest.TestCase, Util):
         self.assertRaises(ValueError, count_n, a, -1) # n < 0
         self.assertRaises(ValueError, count_n, a, 25) # n > a.count()
         self.assertRaises(ValueError, count_n, a, 34) # n > len(a)
-        for n in range(0, 25):
+        for n in range(25):
             self.check_result(a, n, count_n(a, n))
 
     def test_ones(self):
@@ -481,6 +499,7 @@ class TestsCount_N(unittest.TestCase, Util):
         a = bitarray(n)
         a.setall(1)
         self.assertEqual(count_n(a, n), n)
+        self.assertRaises(ValueError, count_n, a, 1, 0)
         self.assertRaises(ValueError, count_n, a, n + 1)
         for _ in range(20):
             i = randint(0, n)
@@ -500,24 +519,28 @@ class TestsCount_N(unittest.TestCase, Util):
             self.assertRaises(ValueError, count_n, a, 2)
 
     def test_large(self):
-        for N in list(range(50)) + [randint(50, 250000) for _ in range(10)]:
+        for _ in range(100):
+            N = randint(100000, 250000)
             a = bitarray(N)
             v = randint(0, 1)
             a.setall(not v)
-            for _ in range(randint(0, min(N, 100))):
+            for _ in range(randint(0, 100)):
                 a[randint(0, N - 1)] = v
-            tc = a.count()      # total count
-            self.check_result(a, tc, count_n(a, tc))
-            self.assertRaises(ValueError, count_n, a, tc + 1)
+            tc = a.count(v)      # total count
+            i = count_n(a, tc, v)
+            self.check_result(a, tc, i, v)
+            self.assertRaises(ValueError, count_n, a, tc + 1, v)
             for _ in range(20):
                 n = randint(0, tc)
-                self.check_result(a, n, count_n(a, n))
+                i = count_n(a, n, v)
+                self.check_result(a, n, i, v)
 
     def test_random(self):
         for a in self.randombitarrays():
-            n = a.count() // 2
-            i = count_n(a, n)
-            self.check_result(a, n, i)
+            for v in 0, 1:
+                n = a.count(v) // 2
+                i = count_n(a, n, v)
+                self.check_result(a, n, i, v)
 
 tests.append(TestsCount_N)
 
@@ -530,7 +553,7 @@ class TestsBitwiseCount(unittest.TestCase, Util):
         ones.setall(1)
         zeros = bitarray(8)
         zeros.setall(0)
-        for i in range(0, 256):
+        for i in range(256):
             a = bitarray()
             a.frombytes(bytes(bytearray([i])))
             cnt = a.count()
@@ -603,36 +626,39 @@ class TestsSubset(unittest.TestCase, Util):
         self.assertRaises(TypeError, subset, a, 4)
         b.append(1)
         self.assertRaises(ValueError, subset, a, b)
+        self.assertRaises(ValueError, subset,
+                          bitarray('01', 'little'),
+                          bitarray('11', 'big'))
 
-    def subset_simple(self, a, b):
-        return (a & b).count() == a.count()
+    def check(self, a, b, res):
+        r = subset(a, b)
+        self.assertIsInstance(r, bool)
+        self.assertEqual(r, res)
+        self.assertEqual(a | b == b, res)
+        self.assertEqual(a & b == a, res)
 
     def test_True(self):
         for a, b in [('', ''), ('0', '1'), ('0', '0'), ('1', '1'),
                      ('000', '111'), ('0101', '0111'),
                      ('000010111', '010011111')]:
-            a, b = bitarray(a), bitarray(b)
-            self.assertTrue(subset(a, b) is True)
-            self.assertTrue(self.subset_simple(a, b) is True)
+            self.check(bitarray(a), bitarray(b), True)
 
     def test_False(self):
         for a, b in [('1', '0'), ('1101', '0111'),
                      ('0000101111', '0100111011')]:
-            a, b = bitarray(a), bitarray(b)
-            self.assertTrue(subset(a, b) is False)
-            self.assertTrue(self.subset_simple(a, b) is False)
+            self.check(bitarray(a), bitarray(b), False)
 
     def test_random(self):
         for a in self.randombitarrays(start=1):
             b = a.copy()
             # we set one random bit in b to 1, so a is always a subset of b
-            b[randint(0, len(a) - 1)] = 1
-            self.assertTrue(subset(a, b))
-            # but b in not always a subset of a
-            self.assertEqual(subset(b, a), self.subset_simple(b, a))
+            b[randint(0, len(a) - 1)] == 1
+            self.check(a, b, True)
+            # but b is only a subset when they are equal
+            self.check(b, a, a == b)
             # we set all bits in a, which ensures that b is a subset of a
             a.setall(1)
-            self.assertTrue(subset(b, a))
+            self.check(b, a, True)
 
 tests.append(TestsSubset)
 
@@ -820,12 +846,17 @@ class TestsBase(unittest.TestCase, Util):
         a = bitarray()
         self.assertRaises(TypeError, ba2base, None, a)
         self.assertRaises(TypeError, base2ba, None, '')
+        self.assertRaises(TypeError, ba2base, 16.0, a)
+        self.assertRaises(TypeError, base2ba, 16.0, '')
         for i in range(-10, 260):
             if i in (2, 4, 8, 16, 32, 64):
                 continue
             self.assertRaises(ValueError, ba2base, i, a)
             self.assertRaises(ValueError, base2ba, i, '')
 
+        self.assertRaises(TypeError, ba2base, 32, None)
+        self.assertRaises(TypeError, base2ba, 32, None)
+
     def test_binary(self):
         a = base2ba(2, '1011')
         self.assertEqual(a, bitarray('1011'))
@@ -948,16 +979,23 @@ class VLFTests(unittest.TestCase, Util):
     def test_decode_args(self):
         if sys.version_info[0] == 3:
             self.assertRaises(TypeError, vl_decode, 'foo')
+            # item not integer
             self.assertRaises(TypeError, vl_decode, iter([b'\x40']))
 
         self.assertRaises(TypeError, vl_decode, b'\x40', 'big', 3)
         self.assertRaises(ValueError, vl_decode, b'\x40', 'foo')
+        # these objects are not iterable
+        for arg in None, 0, 1, 0.0:
+            self.assertRaises(TypeError, vl_decode, arg)
+        # these items cannot be interpreted as ints
         for item in None, 2.34, Ellipsis:
-            self.assertRaises(TypeError, vl_decode, iter([item]))
+            self.assertRaises(TypeError, vl_decode, iter([0x95, item]))
 
-        lst = [b'\xd3\x20', iter(b'\xd3\x20')]
+        b = b'\xd3\x20'
+        lst = [b, iter(b), memoryview(b)]
         if sys.version_info[0] == 3:
             lst.append(iter([0xd3, 0x20]))
+            lst.append(bytearray(b))
         for s in lst:
             a = vl_decode(s, endian=self.random_endian())
             self.assertIsInstance(a, bitarray)
@@ -1017,9 +1055,8 @@ class VLFTests(unittest.TestCase, Util):
                 self.assertEqual(m.group(1), str(n))
             self.assertTrue(a is None)
 
+    @skipIf(sys.version_info[0] == 2)
     def test_decode_invalid_stream(self):
-        if sys.version_info[0] == 2:
-            return
         N = 100
         s = iter(N * (3 * [0x80] + ['XX']) + ['end.'])
         for _ in range(N):
@@ -1043,16 +1080,16 @@ class VLFTests(unittest.TestCase, Util):
         b = vl_decode(s)
         self.check_obj(b)
         self.assertEqual(a, b)
-        PADBITS = 3
-        self.assertEqual(len(s), (len(a) + PADBITS + 6) // 7)
+        LEN_PAD_BITS = 3
+        self.assertEqual(len(s), (len(a) + LEN_PAD_BITS + 6) // 7)
 
         head = ord(s[0]) if sys.version_info[0] == 2 else s[0]
         padding = (head & 0x70) >> 4
-        self.assertEqual(len(a) + padding, 7 * len(s) - PADBITS)
+        self.assertEqual(len(a) + padding, 7 * len(s) - LEN_PAD_BITS)
 
     def test_range(self):
         for n in range(500):
-            self.round_trip(bitarray(n))
+            self.round_trip(urandom(n))
 
     def test_large(self):
         a = urandom(randint(50000, 100000))
@@ -1083,6 +1120,28 @@ class TestsIntegerization(unittest.TestCase, Util):
         # ensure original object wasn't altered
         self.assertEQUAL(a, b)
 
+    def test_ba2int_frozen(self):
+        for a in self.randombitarrays(start=1):
+            b = frozenbitarray(a)
+            self.assertEqual(ba2int(b), ba2int(a))
+            self.assertEQUAL(a, b)
+
+    def test_ba2int_random(self):
+        for a in self.randombitarrays(start=1):
+            b = bitarray(a, 'big')
+            self.assertEqual(a, b)
+            self.assertEqual(ba2int(b), int(b.to01(), 2))
+
+    def test_ba2int_bytes(self):
+        for n in range(1, 50):
+            a = urandom(8 * n, self.random_endian())
+            c = bytearray(a.tobytes())
+            i = 0
+            for x in (c if a.endian() == 'big' else reversed(c)):
+                i <<= 8
+                i |= x
+            self.assertEqual(ba2int(a), i)
+
     def test_int2ba(self):
         self.assertEqual(int2ba(0), bitarray('0'))
         self.assertEqual(int2ba(1), bitarray('1'))
@@ -1118,24 +1177,25 @@ class TestsIntegerization(unittest.TestCase, Util):
                 ('11110',  15),
                 ('00001', -16),
                 ('11111',  -1),
-                ('000000000',    0),
-                ('111111110',  255),
-                ('000000001', -256),
-                ('111111111',   -1),
-                ('0000000000000000000000', 0),
-                ('1001000011000000100010', 9 + 3 * 256 + 17 * 2 ** 16),
-                ('1111111111111111111110', 2 ** 21 - 1),
-                ('0000000000000000000001', -2 ** 21),
-                ('1001000011000000100011', -2 ** 21
+                ('00000000 0',    0),
+                ('11111111 0',  255),
+                ('00000000 1', -256),
+                ('11111111 1',   -1),
+                ('00000000 00000000 000000', 0),
+                ('10010000 11000000 100010', 9 + 3 * 256 + 17 * 2 ** 16),
+                ('11111111 11111111 111110', 2 ** 21 - 1),
+                ('00000000 00000000 000001', -2 ** 21),
+                ('10010000 11000000 100011', -2 ** 21
                                            + (9 + 3 * 256 + 17 * 2 ** 16)),
-                ('1111111111111111111111', -1),
+                ('11111111 11111111 111111', -1),
         ]:
             self.assertEqual(ba2int(bitarray(s, 'little'), signed=1), i)
             self.assertEqual(ba2int(bitarray(s[::-1], 'big'), signed=1), i)
 
-            self.assertEQUAL(int2ba(i, len(s), 'little', signed=1),
+            len_s = len(bitarray(s))
+            self.assertEQUAL(int2ba(i, len_s, 'little', signed=1),
                              bitarray(s, 'little'))
-            self.assertEQUAL(int2ba(i, len(s), 'big', signed=1),
+            self.assertEQUAL(int2ba(i, len_s, 'big', signed=1),
                              bitarray(s[::-1], 'big'))
 
     def test_int2ba_overflow(self):
@@ -1254,9 +1314,8 @@ class MixedTests(unittest.TestCase, Util):
             self.assertEqual(t, s)
             self.assertEqual(eval(t), i)
 
+    @skipIf(sys.version_info[0] == 2)
     def test_oct(self):
-        if sys.version_info[0] == 2:
-            return
         for i in range(1000):
             s = oct(i)
             self.assertEqual(s[:2], '0o')
@@ -1322,6 +1381,23 @@ class MixedTests(unittest.TestCase, Util):
                 c <<= n
                 self.assertEqual(ba2int(c), i << n)
 
+    def test_primes(self):  # Sieve of Eratosthenes
+        sieve = bitarray(10000)
+        sieve.setall(1)
+        sieve[:2] = 0  # zero and one are not prime
+        for i in range(2, 100):
+            if sieve[i]:
+                sieve[i * i::i] = 0
+        # the first 15 primes
+        self.assertEqual(sieve.search(1, 15), [2, 3, 5, 7, 11, 13, 17, 19,
+                                               23, 29, 31, 37, 41, 43, 47])
+        # there are 1229 primes between 1 and 10000
+        self.assertEqual(sieve.count(1), 1229)
+        # there are 119 primes between 4000 and 5000
+        self.assertEqual(sieve.count(1, 4000, 5000), 119)
+        # the 1000th prime is 7919
+        self.assertEqual(count_n(sieve, 1000) - 1, 7919)
+
 tests.append(MixedTests)
 
 # ---------------------------------------------------------------------------
@@ -1342,36 +1418,65 @@ class TestsSerialization(unittest.TestCase, Util):
 
     def test_zeros_and_ones(self):
         for endian in 'little', 'big':
-            for n in range(100):
+            for n in range(50):
                 a = zeros(n, endian)
                 s = serialize(a)
                 self.assertIsInstance(s, bytes)
-                self.assertEqual(s[1:], b'\0' * bits2bytes(n))
+                self.assertEqual(s[1:], b'\0' * a.nbytes)
                 self.assertEQUAL(a, deserialize(s))
                 a.setall(1)
                 self.assertEQUAL(a, deserialize(serialize(a)))
 
-    def test_wrong_args(self):
-        self.assertRaises(TypeError, serialize, '0')
+    def test_serialize_args(self):
+        for x in '0', 0, 1, b'\x00', 0.0, [0, 1], bytearray([0]):
+            self.assertRaises(TypeError, serialize, x)
+        # no arguments
+        self.assertRaises(TypeError, serialize)
+        # too many arguments
         self.assertRaises(TypeError, serialize, bitarray(), 1)
-        self.assertRaises(TypeError, deserialize, u'01')
+
+    # On Python 2, bytes(x) is the string representation of x,
+    # so x can almost be of any type.  And the string representation
+    # of these object will cause ValueErrors in bitarray(x).
+    # Instead of adding special cases in deserialize() or here,
+    # we decided to skip these tests for Python 2 altogether.
+    @skipIf(sys.version_info[0] == 2)
+    def test_deserialize_args(self):
+        for x in 0, 1, False, True, None, u'', u'01', 0.0, [0, 0.6]:
+            self.assertRaises(TypeError, deserialize, x)
         self.assertRaises(TypeError, deserialize, b'\x00', 1)
+        self.assertRaises(ValueError, deserialize, [0, 256])
+
+        b = b'\x03\x06'
+        for s in b, bytearray(b), memoryview(b), iter(b):
+            a = deserialize(s)
+            self.assertEqual(a.endian(), 'little')
+            self.assertEqual(a, bitarray('01100'))
 
     def test_invalid_bytes(self):
         self.assertRaises(ValueError, deserialize, b'')
 
+        def check_msg(b):
+            # Python 2: PyErr_Format() seems to handle "0x%02x"
+            # incorrectly.  E.g. instead of "0x01", I get "0x1"
+            if sys.version_info[0] == 3:
+                msg = "invalid header byte: 0x%02x" % b[0]
+                self.assertRaisesMessage(ValueError, msg, deserialize, b)
+
         for i in range(256):
             b = bytes(bytearray([i]))
             if i == 0 or i == 16:
                 self.assertEqual(deserialize(b), bitarray())
             else:
                 self.assertRaises(ValueError, deserialize, b)
+                check_msg(b)
 
             b += b'\0'
             if i < 32 and i % 16 < 8:
                 self.assertEqual(deserialize(b), zeros(8 - i % 8))
             else:
                 self.assertRaises(ValueError, deserialize, b)
+                check_msg(b)
 
     def test_bits_ignored(self):
         # the unused padding bits (with the last bytes) are ignored
@@ -1405,16 +1510,6 @@ class TestsHuffman(unittest.TestCase):
         self.assertEqual(len(code['as']), 2)
         self.assertEqual(len(code[None]), 2)
 
-    def test_tiny(self):
-        code = huffman_code({0: 0})
-        self.assertEqual(len(code), 1)
-        self.assertEqual(code, {0: bitarray()})
-
-        code = huffman_code({0: 0, 1: 0})
-        self.assertEqual(len(code), 2)
-        for i in range(2):
-            self.assertEqual(len(code[i]), 1)
-
     def test_endianness(self):
         freq = {'A': 10, 'B': 2, 'C': 5}
         for endian in 'big', 'little':
@@ -1429,14 +1524,31 @@ class TestsHuffman(unittest.TestCase):
         self.assertRaises(TypeError, huffman_code, None)
         # cannot compare 'a' with 1
         self.assertRaises(TypeError, huffman_code, {'A': 'a', 'B': 1})
+        # frequency map cannot be empty
         self.assertRaises(ValueError, huffman_code, {})
 
+    def test_one_symbol(self):
+        cnt = {'a': 1}
+        code = huffman_code(cnt)
+        self.assertEqual(code, {'a': bitarray('0')})
+        for n in range(4):
+            msg = n * ['a']
+            a = bitarray()
+            a.encode(code, msg)
+            self.assertEqual(a.to01(), n * '0')
+            self.assertEqual(a.decode(code), msg)
+            a.append(1)
+            self.assertRaises(ValueError, a.decode, code)
+            self.assertRaises(ValueError, list, a.iterdecode(code))
+
     def check_tree(self, code):
         n = len(code)
         tree = decodetree(code)
         self.assertEqual(tree.todict(), code)
         # ensure tree has 2n-1 nodes (n symbol nodes and n-1 internal nodes)
         self.assertEqual(tree.nodes(), 2 * n - 1)
+        # a proper Huffman tree is complete
+        self.assertTrue(tree.complete())
 
     def test_balanced(self):
         n = 6
@@ -1476,15 +1588,275 @@ class TestsHuffman(unittest.TestCase):
         self.check_tree(code)
 
     def test_random_freq(self):
-        N = randint(2, 1000)
-        # create Huffman code for N symbols
-        code = huffman_code({i: random() for i in range(N)})
-        self.check_tree(code)
+        for n in 2, 3, 5, randint(50, 200):
+            # create Huffman code for n symbols
+            code = huffman_code({i: random() for i in range(n)})
+            self.check_tree(code)
 
 tests.append(TestsHuffman)
 
 # ---------------------------------------------------------------------------
 
+class TestsCanonicalHuffman(unittest.TestCase, Util):
+
+    def test_basic(self):
+        plain = bytearray(b'the quick brown fox jumps over the lazy dog.')
+        chc, count, symbol = canonical_huffman(Counter(plain))
+        self.assertIsInstance(chc, dict)
+        self.assertIsInstance(count, list)
+        self.assertIsInstance(symbol, list)
+        a = bitarray()
+        a.encode(chc, plain)
+        self.assertEqual(bytearray(a.iterdecode(chc)), plain)
+        self.assertEqual(bytearray(canonical_decode(a, count, symbol)), plain)
+
+    def test_canonical_huffman_errors(self):
+        self.assertRaises(TypeError, canonical_huffman, [])
+        # frequency map cannot be empty
+        self.assertRaises(ValueError, canonical_huffman, {})
+        self.assertRaises(TypeError, canonical_huffman)
+        cnt = huffman_code(Counter('aabc'))
+        self.assertRaises(TypeError, canonical_huffman, cnt, 'a')
+
+    def test_one_symbol(self):
+        cnt = {'a': 1}
+        chc, count, symbol = canonical_huffman(cnt)
+        self.assertEqual(chc, {'a': bitarray('0')})
+        self.assertEqual(count, [0, 1])
+        self.assertEqual(symbol, ['a'])
+        for n in range(4):
+            msg = n * ['a']
+            a = bitarray()
+            a.encode(chc, msg)
+            self.assertEqual(a.to01(), n * '0')
+            self.assertEqual(list(canonical_decode(a, count, symbol)), msg)
+            a.append(1)
+            self.assertRaises(ValueError, list,
+                              canonical_decode(a, count, symbol))
+
+    def test_canonical_decode_errors(self):
+        a = bitarray('1101')
+        s = ['a']
+        # bitarray not of bitarray type
+        self.assertRaises(TypeError, canonical_decode, '11', [0, 1], s)
+        # count not sequence
+        self.assertRaises(TypeError, canonical_decode, a, {0, 1}, s)
+        # count element not an int
+        self.assertRaises(TypeError, canonical_decode, a, [0, 1.0], s)
+        # count element overflow
+        self.assertRaises(OverflowError, canonical_decode, a, [0, 1 << 65], s)
+        # negative count
+        self.assertRaises(ValueError, canonical_decode, a, [0, -1], s)
+        # count list too long
+        self.assertRaises(ValueError, canonical_decode, a, 32 * [0], s)
+        # symbol not sequence
+        self.assertRaises(TypeError, canonical_decode, a, [0, 1], 43)
+
+        symbol = ['a', 'b', 'c', 'd']
+        # sum(count) != len(symbol)
+        self.assertRaisesMessage(ValueError,
+                                 "sum(count) = 3, but len(symbol) = 4",
+                                 canonical_decode, a, [0, 1, 2], symbol)
+        # count[i] > 1 << i
+        self.assertRaisesMessage(ValueError,
+                        "count[2] cannot be negative or larger than 4, got 5",
+                        canonical_decode, a, [0, 2, 5], symbol)
+
+    def test_canonical_decode_simple(self):
+        # symbols can be anything, they do not even have to be hashable here
+        cnt = [0, 0, 4]
+        s = ['A', 42, [1.2-3.7j, 4j], {'B': 6}]
+        a = bitarray('00 01 10 11')
+        # count can be a list
+        self.assertEqual(list(canonical_decode(a, cnt, s)), s)
+        # count can also be a tuple (any sequence object in fact)
+        self.assertEqual(list(canonical_decode(a, (0, 0, 4), s)), s)
+        self.assertEqual(list(canonical_decode(7 * a, cnt, s)), 7 * s)
+        # the count list may have extra 0's at the end (but not too many)
+        count = [0, 0, 4, 0, 0, 0, 0, 0]
+        self.assertEqual(list(canonical_decode(a, count, s)), s)
+        # the element count[0] is unused
+        self.assertEqual(list(canonical_decode(a, [-47, 0, 4], s)), s)
+        # in fact it can be anything, as it is entirely ignored
+        self.assertEqual(list(canonical_decode(a, [s, 0, 4], s)), s)
+
+        # the symbol argument can be any sequence object
+        s = [65, 66, 67, 98]
+        self.assertEqual(list(canonical_decode(a, cnt, s)), s)
+        self.assertEqual(list(canonical_decode(a, cnt, bytearray(s))), s)
+        self.assertEqual(list(canonical_decode(a, cnt, tuple(s))), s)
+        if sys.version_info[0] == 3:
+            self.assertEqual(list(canonical_decode(a, cnt, bytes(s))), s)
+        # Implementation Note:
+        #   The symbol can even be an iterable.  This was done because we
+        #   want to use PySequence_Fast in order to convert sequence
+        #   objects (like bytes and bytearray) to a list.  This is faster
+        #   as all objects are now elements in an array of pointers (as
+        #   opposed to having the object's __getitem__ method called on
+        #   every iteration).
+        self.assertEqual(list(canonical_decode(a, cnt, iter(s))), s)
+
+    def test_canonical_decode_empty(self):
+        a = bitarray()
+        # count and symbol are empty, ok because sum([]) == len([])
+        self.assertEqual(list(canonical_decode(a, [], [])), [])
+        a.append(0)
+        self.assertRaisesMessage(ValueError, "reached end of bitarray",
+                                 list, canonical_decode(a, [], []))
+        a = bitarray(31 * '0')
+        self.assertRaisesMessage(ValueError, "ran out of codes",
+                                 list, canonical_decode(a, [], []))
+
+    def test_canonical_decode_one_symbol(self):
+        symbols = ['A']
+        count = [0, 1]
+        a = bitarray('000')
+        self.assertEqual(list(canonical_decode(a, count, symbols)),
+                         3 * symbols)
+        a.append(1)
+        a.extend(bitarray(10 * '0'))
+        iterator = canonical_decode(a, count, symbols)
+        self.assertRaisesMessage(ValueError, "reached end of bitarray",
+                                 list, iterator)
+
+        a.extend(bitarray(20 * '0'))
+        iterator = canonical_decode(a, count, symbols)
+        self.assertRaisesMessage(ValueError, "ran out of codes",
+                                 list, iterator)
+
+    def test_canonical_decode_large(self):
+        with open(__file__, 'rb') as f:
+            msg = bytearray(f.read())
+        self.assertTrue(len(msg) > 50000)
+        codedict, count, symbol = canonical_huffman(Counter(msg))
+        a = bitarray()
+        a.encode(codedict, msg)
+        self.assertEqual(bytearray(canonical_decode(a, count, symbol)), msg)
+        self.check_code(codedict, count, symbol)
+
+    def test_canonical_decode_symbol_change(self):
+        msg = bytearray(b"Hello World!")
+        codedict, count, symbol = canonical_huffman(Counter(msg))
+        self.check_code(codedict, count, symbol)
+        a = bitarray()
+        a.encode(codedict, 10 * msg)
+
+        it = canonical_decode(a, count, symbol)
+        def decode_one_msg():
+            return bytearray(next(it) for _ in range(len(msg)))
+
+        self.assertEqual(decode_one_msg(), msg)
+        symbol[symbol.index(ord("l"))] = ord("k")
+        self.assertEqual(decode_one_msg(), bytearray(b"Hekko Workd!"))
+        del symbol[:]
+        self.assertRaises(IndexError, decode_one_msg)
+
+    def ensure_sorted(self, chc, symbol):
+        # ensure codes are sorted
+        for i in range(len(symbol) - 1):
+            a = chc[symbol[i]]
+            b = chc[symbol[i + 1]]
+            self.assertTrue(ba2int(a) < ba2int(b))
+
+    def ensure_consecutive(self, chc, count, symbol):
+        first = 0
+        for nbits, cnt in enumerate(count):
+            for i in range(first, first + cnt - 1):
+                # ensure two consecutive codes (with same bit length) have
+                # consecutive integer values
+                a = chc[symbol[i]]
+                b = chc[symbol[i + 1]]
+                self.assertTrue(len(a) == len(b) == nbits)
+                self.assertEqual(ba2int(a) + 1, ba2int(b))
+            first += cnt
+
+    def ensure_count(self, chc, count):
+        # ensure count list corresponds to length counts from codedict
+        maxbits = max(len(a) for a in chc.values())
+        my_count = (maxbits + 1) * [0]
+        for a in chc.values():
+            self.assertEqual(a.endian(), 'big')
+            my_count[len(a)] += 1
+        self.assertEqual(my_count, list(count))
+
+    def ensure_complete(self, count):
+        # ensure code is complete and not oversubscribed
+        maxbits = len(count)
+        x = sum(count[i] << (maxbits - i) for i in range(1, maxbits))
+        self.assertEqual(x, 1 << maxbits)
+
+    def ensure_complete_2(self, chc):
+        # ensure code is complete
+        dt = decodetree(chc)
+        self.assertTrue(dt.complete())
+
+    def ensure_round_trip(self, chc, count, symbol):
+        # create a short test message, encode and decode
+        msg = [choice(symbol) for _ in range(10)]
+        a = bitarray()
+        a.encode(chc, msg)
+        it = canonical_decode(a, count, symbol)
+        # the iterator holds a reference to the bitarray and symbol list
+        del a, count, symbol
+        self.assertEqual(type(it).__name__, 'canonical_decodeiter')
+        self.assertEqual(list(it), msg)
+
+    def check_code(self, chc, count, symbol):
+        self.assertTrue(len(chc) == len(symbol) == sum(count))
+        self.assertEqual(count[0], 0)  # no codes have length 0
+        self.assertTrue(set(chc) == set(symbol))
+        # the code of the last symbol has all 1 bits
+        self.assertTrue(chc[symbol[-1]].all())
+        # the code of the first symbol starts with bit 0
+        self.assertFalse(chc[symbol[0]][0])
+
+        self.ensure_sorted(chc, symbol)
+        self.ensure_consecutive(chc, count, symbol)
+        self.ensure_count(chc, count)
+        self.ensure_complete(count)
+        self.ensure_complete_2(chc)
+        self.ensure_round_trip(chc, count, symbol)
+
+    def test_simple_counter(self):
+        plain = bytearray(b'the quick brown fox jumps over the lazy dog.')
+        cnt = Counter(plain)
+        code, count, symbol = canonical_huffman(cnt)
+        self.check_code(code, count, symbol)
+        self.check_code(code, tuple(count), tuple(symbol))
+        self.check_code(code, bytearray(count), symbol)
+        self.check_code(code, count, bytearray(symbol))
+
+    def test_balanced(self):
+        n = 7
+        freq = {}
+        for i in range(2 ** n):
+            freq[i] = 1
+        code, count, sym = canonical_huffman(freq)
+        self.assertEqual(len(code), 2 ** n)
+        self.assertTrue(all(len(v) == n for v in code.values()))
+        self.check_code(code, count, sym)
+
+    def test_unbalanced(self):
+        n = 29
+        freq = {}
+        for i in range(n):
+            freq[i] = 2 ** i
+        code = canonical_huffman(freq)[0]
+        self.assertEqual(len(code), n)
+        for i in range(n):
+            self.assertEqual(len(code[i]), n - (1 if i <= 1 else i))
+        self.check_code(*canonical_huffman(freq))
+
+    def test_random_freq(self):
+        for n in 2, 3, 5, randint(50, 200):
+            freq = {i: random() for i in range(n)}
+            self.check_code(*canonical_huffman(freq))
+
+
+tests.append(TestsCanonicalHuffman)
+
+# ---------------------------------------------------------------------------
+
 def run(verbosity=1):
     import bitarray
 
